/*
	AEFlame - After Effects version of flame
	Original version of flame by Scott Draves <spot@cs.cmu.edu>
	After Effects version by Andrew B. Davidson <aeflame@mindspring.com> 
    flame Copyright (C) 1992 Scott Draves.  
    AEFlame Copyright (C) 1999 Andrew B. Davidson.  

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#pragma once

#ifndef __AEFLAME_H__
#define __AEFLAME_H__

#include "AE_Effect.h"
#include "AE_EffectCB.h"

#ifdef MSWindows
#include <Windows.h>
#endif

#define	AEF_NAME			"AEFlame"
#define AEF_DESCRIPTION		"Copyright (C) 1999 Andrew B. Davidson. <aeflame@mindspring.com>\r"	\
							"Original flame (C) 1992 Scott Draves.\r" \
							"Distributed under the GNU General Public License.\r" \
							"ABSOLUTELY NO WARRANTY. See file COPYING.txt."

#define	AEF_MAJOR_VERSION		1
#define	AEF_MINOR_VERSION		0
#define	AEF_BUG_VERSION			0
#define	AEF_STAGE_VERSION		PF_Stage_DEVELOP
#define	AEF_BUILD_VERSION		0

#define kNumImageChannels 4		// 3 = RGB; 4 = RGBA; other values invalid
#define kNumXforms 4			// ABDTODO FIX THIS to 6 when AE lets us...
#define kNumCoefficients 6		// don't change this
#define kNumVariations 7		// don't change this

// parameter list
enum {
	AEF_DEFAULT_INPUT_LAYER = 0,	// default input layer 

	AEF_COMMAND_MENU,
	AEF_RANDOMIZE_COMMAND_MENU,
	AEF_RANDOM_VECTOR_LENGTH,

	AEF_GLOBAL_PARAMETERS_START,	// GLOBAL PARAMETERS
		AEF_LOAD_PARAMETERS,
		AEF_CENTER_OFFSET_X,
		AEF_CENTER_OFFSET_Y,
		AEF_ALPHA_BLEND,
		AEF_SHOW_ALPHA_ONLY,
		AEF_BLEND,
		AEF_PIXELS_PER_UNIT,
		AEF_ZOOM,
		AEF_SPATIAL_OVERSAMPLE,
		AEF_SPATIAL_FILTER_RADIUS,
		AEF_SAMPLE_DENSITY,
		AEF_GAMMA,
		AEF_CONTRAST,
		AEF_BRIGHTNESS,
		AEF_WHITE_LEVEL,
/*		AEF_RANDOM_PERIOCITY,
*/		AEF_COLORMAP,
		AEF_COLORMAP_ROTATION,
		AEF_COLORMAP_INVERT,
		AEF_HUE_ROTATION,
		AEF_COLORMAP_INTERPOLATE,
		AEF_NUM_BATCHES,
		AEF_NUM_STRIPS,
		AEF_RANDOM_SEED,
	AEF_GLOBAL_PARAMETERS_END,

	AEF_XFORM0_COMMAND_MENU,

	AEF_XFORM0_START,				// start of this xform section...
// ALWAYS ENABLED		AEF_XFORM0_ENABLED,			// BOOLEAN  master on/off for this xform
		AEF_XFORM0_DENSITY,
		AEF_XFORM0_COLOR,
		AEF_XFORM0_VAR0,			// percentage of each variation to use (7 per xform)
		AEF_XFORM0_VAR1,
		AEF_XFORM0_VAR2,
		AEF_XFORM0_VAR3,
		AEF_XFORM0_VAR4,
		AEF_XFORM0_VAR5,
		AEF_XFORM0_VAR6,
		AEF_XFORM0_COEF0,			// actual coefficients (6 per AEF_XFORM)
		AEF_XFORM0_COEF1,
		AEF_XFORM0_COEF2,
		AEF_XFORM0_COEF3,
		AEF_XFORM0_COEF4,
		AEF_XFORM0_COEF5,
	AEF_XFORM0_END,					// end of this AEF_XFORM section...

	AEF_XFORM1_COMMAND_MENU,

	AEF_XFORM1_START,				// start of this AEF_XFORM section...
// ALWAYS ENABLED		AEF_XFORM1_ENABLED,			// BOOLEAN  master on/off for this AEF_XFORM
		AEF_XFORM1_DENSITY,
		AEF_XFORM1_COLOR,
		AEF_XFORM1_VAR0,			// percentage of each variation to use (7 per AEF_XFORM)
		AEF_XFORM1_VAR1,
		AEF_XFORM1_VAR2,
		AEF_XFORM1_VAR3,
		AEF_XFORM1_VAR4,
		AEF_XFORM1_VAR5,
		AEF_XFORM1_VAR6,
		AEF_XFORM1_COEF0,			// actual coefficients (6 per AEF_XFORM)
		AEF_XFORM1_COEF1,
		AEF_XFORM1_COEF2,
		AEF_XFORM1_COEF3,
		AEF_XFORM1_COEF4,
		AEF_XFORM1_COEF5,
	AEF_XFORM1_END,					// end of this AEF_XFORM section...

#if (kNumXforms > 2)
	AEF_XFORM2_COMMAND_MENU,

	AEF_XFORM2_START,				// start of this AEF_XFORM section...
		AEF_XFORM2_ENABLED,			// BOOLEAN  master on/off for this AEF_XFORM
		AEF_XFORM2_DENSITY,
		AEF_XFORM2_COLOR,
		AEF_XFORM2_VAR0,			// percentage of each variation to use (7 per AEF_XFORM)
		AEF_XFORM2_VAR1,
		AEF_XFORM2_VAR2,
		AEF_XFORM2_VAR3,
		AEF_XFORM2_VAR4,
		AEF_XFORM2_VAR5,
		AEF_XFORM2_VAR6,
		AEF_XFORM2_COEF0,			// actual coefficients (6 per AEF_XFORM)
		AEF_XFORM2_COEF1,
		AEF_XFORM2_COEF2,
		AEF_XFORM2_COEF3,
		AEF_XFORM2_COEF4,
		AEF_XFORM2_COEF5,
	AEF_XFORM2_END,					// end of this AEF_XFORM section...
#endif
#if (kNumXforms > 3)
	AEF_XFORM3_COMMAND_MENU,

	AEF_XFORM3_START,				// start of this AEF_XFORM section...
		AEF_XFORM3_ENABLED,			// BOOLEAN  master on/off for this AEF_XFORM
		AEF_XFORM3_DENSITY,
		AEF_XFORM3_COLOR,
		AEF_XFORM3_VAR0,			// percentage of each variation to use (7 per AEF_XFORM)
		AEF_XFORM3_VAR1,
		AEF_XFORM3_VAR2,
		AEF_XFORM3_VAR3,
		AEF_XFORM3_VAR4,
		AEF_XFORM3_VAR5,
		AEF_XFORM3_VAR6,
		AEF_XFORM3_COEF0,			// actual coefficients (6 per AEF_XFORM)
		AEF_XFORM3_COEF1,
		AEF_XFORM3_COEF2,
		AEF_XFORM3_COEF3,
		AEF_XFORM3_COEF4,
		AEF_XFORM3_COEF5,
	AEF_XFORM3_END,					// end of this AEF_XFORM section...
#endif
#if (kNumXforms > 4)
	AEF_XFORM4_COMMAND_MENU,

	AEF_XFORM4_START,				// start of this AEF_XFORM section...
		AEF_XFORM4_ENABLED,			// BOOLEAN  master on/off for this AEF_XFORM
		AEF_XFORM4_DENSITY,
		AEF_XFORM4_COLOR,
		AEF_XFORM4_VAR0,			// percentage of each variation to use (7 per AEF_XFORM)
		AEF_XFORM4_VAR1,
		AEF_XFORM4_VAR2,
		AEF_XFORM4_VAR3,
		AEF_XFORM4_VAR4,
		AEF_XFORM4_VAR5,
		AEF_XFORM4_VAR6,
		AEF_XFORM4_COEF0,			// actual coefficients (6 per AEF_XFORM)
		AEF_XFORM4_COEF1,
		AEF_XFORM4_COEF2,
		AEF_XFORM4_COEF3,
		AEF_XFORM4_COEF4,
		AEF_XFORM4_COEF5,
	AEF_XFORM4_END,					// end of this AEF_XFORM section...
#endif
#if (kNumXforms > 5)
	AEF_XFORM5_COMMAND_MENU,

	AEF_XFORM5_START,				// start of this AEF_XFORM section...
		AEF_XFORM5_ENABLED,			// BOOLEAN  master on/off for this AEF_XFORM
		AEF_XFORM5_DENSITY,
		AEF_XFORM5_COLOR,
		AEF_XFORM5_VAR0,			// percentage of each variation to use (7 per AEF_XFORM)
		AEF_XFORM5_VAR1,
		AEF_XFORM5_VAR2,
		AEF_XFORM5_VAR3,
		AEF_XFORM5_VAR4,
		AEF_XFORM5_VAR5,
		AEF_XFORM5_VAR6,
		AEF_XFORM5_COEF0,			// actual coefficients (6 per AEF_XFORM)
		AEF_XFORM5_COEF1,
		AEF_XFORM5_COEF2,
		AEF_XFORM5_COEF3,
		AEF_XFORM5_COEF4,
		AEF_XFORM5_COEF5,
	AEF_XFORM5_END,					// end of this AEF_XFORM section...
#endif

	AEF_NUM_PARAMS
};

PF_Err main (	PF_Cmd			cmd,
				PF_InData		*in_data,
				PF_OutData		*out_data,
				PF_ParamDef		*params[],
				PF_LayerDef		*output,
				void			*extra	);

PF_Err About (	PF_InData		*in_data,
				PF_OutData		*out_data,
				PF_ParamDef		*params[],
				PF_LayerDef		*output );

PF_Err GlobalSetup (PF_InData		*in_data,
					PF_OutData		*out_data,
					PF_ParamDef		*params[],
					PF_LayerDef		*output );

PF_Err GlobalSetdown (	PF_InData		*in_data,
						PF_OutData		*out_data,
						PF_ParamDef		*params[],
						PF_LayerDef		*output );

PF_Err ParamsSetup (PF_InData		*in_data,
					PF_OutData		*out_data,
					PF_ParamDef		*params[],
					PF_LayerDef		*output);

PF_Err Render ( PF_InData		*in_data,
				PF_OutData		*out_data,
				PF_ParamDef		*params[],
				PF_LayerDef		*output );

void HandleMenuCommand(PF_InData		*in_data,
						PF_OutData		*out_data,
						PF_ParamDef		*params[],
						PF_LayerDef		*output,
						PF_UserChangedParamExtra	*extra,
						int iParamIndex,
						int iMenuCommand);

void HandleXformMenuCommand(PF_InData		*in_data,
						PF_OutData		*out_data,
						PF_ParamDef		*params[],
						PF_LayerDef		*output,
						PF_UserChangedParamExtra	*extra,
						int iXformNum,
						int iXformVarNum,
						int iXformCoeffNum,
						int iParamIndex,
						int iMenuCommand);

void HandleRandomizeMenuCommand(PF_InData		*in_data,
						PF_OutData		*out_data,
						PF_ParamDef		*params[],
						PF_LayerDef		*output,
						PF_UserChangedParamExtra	*extra,
						int iParamIndex,
						int iMenuCommand);

PF_Err	HandleChangedParam(	PF_InData		*in_data,
							PF_OutData		*out_data,
							PF_ParamDef		*params[],
							PF_LayerDef		*output,
							PF_UserChangedParamExtra	*extra);

void srandom(unsigned int seed);
double frandom();
long random();

#define CLAMP(a, min, max)  a = (((a) > (max)) ? (max) : (((a) < (min)) ? (min) : (a)))
#define UCHAR2DOUBLE(uchar) ((double)(uchar) / (double)255.0)
#define DOUBLE2UCHAR(dbl) (unsigned char)((dbl) * 255.0)
#define	FIX2FLOAT(F)		((F)*((double)(1/65536.0)))


#define	BLEND_MIN		(0L << 16)	// show just shifted
#define	BLEND_MAX		(1L << 16)	// show just original
#define	BLEND_DFLT		(0L << 16)


void AppendGlobalParamsString(PF_InData	*in_data, PF_ParamDef	*params[], char *s, int iPayAttentionToDownsample);
void AppendXform0String(PF_ParamDef	*params[], char *s);
void AppendXform1String(PF_ParamDef	*params[], char *s);
void AppendXform2String(PF_ParamDef	*params[], char *s);
void AppendXform3String(PF_ParamDef	*params[], char *s);
void AppendXform4String(PF_ParamDef	*params[], char *s);
void AppendXform5String(PF_ParamDef	*params[], char *s);

int GetNumLines(const char *iFullFileName);

// DISK ID's... NEVER EVER DELETE ITEMS FROM THIS LIST!!!
// ***IMPORTANT*** NEVER EVER DELETE ITEMS FROM THIS LIST!!!
// ***IMPORTANT*** NEVER EVER DELETE ITEMS FROM THIS LIST!!!
// ***IMPORTANT*** NEVER EVER DELETE ITEMS FROM THIS LIST!!!
// ***IMPORTANT*** NEVER EVER DELETE ITEMS FROM THIS LIST!!!
// ***IMPORTANT*** NEVER EVER ADD ITEMS EXCEPT TO THE *END* OF THIS LIST!!!
// ***IMPORTANT*** NEVER EVER ADD ITEMS EXCEPT TO THE *END* OF THIS LIST!!!
// ***IMPORTANT*** NEVER EVER ADD ITEMS EXCEPT TO THE *END* OF THIS LIST!!!
// ***IMPORTANT*** NEVER EVER ADD ITEMS EXCEPT TO THE *END* OF THIS LIST!!!
// ***IMPORTANT*** NEVER EVER ADD ITEMS EXCEPT TO THE *END* OF THIS LIST!!!
// these are the codes used to save params to disk that allows us to 
// load files even if later versions of AEFlame have different numbers, types,
// order, etc. of parameters.  But it works *ONLY* if when you are adding
// a parameter, you add a new item at the *END* of this list (*NOT* in the
// middle).  Also, do *NOT* delete items from this list, even if that parameter
// is obsolete or not longer shows up.... 
enum {
	DISKID_DEFAULT_INPUT_LAYER = 0,	// default input layer 

	DISKID_COMMAND_MENU,
	DISKID_RANDOMIZE_COMMAND_MENU,
	DISKID_RANDOM_VECTOR_LENGTH,

	DISKID_GLOBAL_PARAMETERS_START,	// GLOBAL PARAMETERS
		DISKID_LOAD_PARAMETERS,
		DISKID_CENTER_OFFSET_X,
		DISKID_CENTER_OFFSET_Y,
		DISKID_BLEND,
		DISKID_PIXELS_PER_UNIT,
		DISKID_ZOOM,
		DISKID_SPATIAL_OVERSAMPLE,
		DISKID_SPATIAL_FILTER_RADIUS,
		DISKID_SAMPLE_DENSITY,
		DISKID_GAMMA,
		DISKID_CONTRAST,
		DISKID_BRIGHTNESS,
		DISKID_WHITE_LEVEL,
		DISKID_COLORMAP,
		DISKID_COLORMAP_ROTATION,
		DISKID_COLORMAP_INVERT,
		DISKID_COLORMAP_INTERPOLATE,
		DISKID_NUM_BATCHES,
		DISKID_NUM_STRIPS,
		DISKID_RANDOM_SEED,
	DISKID_GLOBAL_PARAMETERS_END,

	DISKID_XFORM0_COMMAND_MENU,

	DISKID_XFORM0_START,				// start of this xform section...
// ALWAYS ENABLED		DISKID_XFORM0_ENABLED,			// BOOLEAN  master on/off for this xform
		DISKID_XFORM0_DENSITY,
		DISKID_XFORM0_COLOR,
		DISKID_XFORM0_VAR0,			// percentage of each variation to use (7 per xform)
		DISKID_XFORM0_VAR1,
		DISKID_XFORM0_VAR2,
		DISKID_XFORM0_VAR3,
		DISKID_XFORM0_VAR4,
		DISKID_XFORM0_VAR5,
		DISKID_XFORM0_VAR6,
		DISKID_XFORM0_COEF0,			// actual coefficients (6 per DISKID_XFORM)
		DISKID_XFORM0_COEF1,
		DISKID_XFORM0_COEF2,
		DISKID_XFORM0_COEF3,
		DISKID_XFORM0_COEF4,
		DISKID_XFORM0_COEF5,
	DISKID_XFORM0_END,					// end of this DISKID_XFORM section...

	DISKID_XFORM1_COMMAND_MENU,

	DISKID_XFORM1_START,				// start of this DISKID_XFORM section...
// ALWAYS ENABLED		DISKID_XFORM1_ENABLED,			// BOOLEAN  master on/off for this DISKID_XFORM
		DISKID_XFORM1_DENSITY,
		DISKID_XFORM1_COLOR,
		DISKID_XFORM1_VAR0,			// percentage of each variation to use (7 per DISKID_XFORM)
		DISKID_XFORM1_VAR1,
		DISKID_XFORM1_VAR2,
		DISKID_XFORM1_VAR3,
		DISKID_XFORM1_VAR4,
		DISKID_XFORM1_VAR5,
		DISKID_XFORM1_VAR6,
		DISKID_XFORM1_COEF0,			// actual coefficients (6 per DISKID_XFORM)
		DISKID_XFORM1_COEF1,
		DISKID_XFORM1_COEF2,
		DISKID_XFORM1_COEF3,
		DISKID_XFORM1_COEF4,
		DISKID_XFORM1_COEF5,
	DISKID_XFORM1_END,					// end of this DISKID_XFORM section...
	DISKID_XFORM2_COMMAND_MENU,

	DISKID_XFORM2_START,				// start of this DISKID_XFORM section...
		DISKID_XFORM2_ENABLED,			// BOOLEAN  master on/off for this DISKID_XFORM
		DISKID_XFORM2_DENSITY,
		DISKID_XFORM2_COLOR,
		DISKID_XFORM2_VAR0,			// percentage of each variation to use (7 per DISKID_XFORM)
		DISKID_XFORM2_VAR1,
		DISKID_XFORM2_VAR2,
		DISKID_XFORM2_VAR3,
		DISKID_XFORM2_VAR4,
		DISKID_XFORM2_VAR5,
		DISKID_XFORM2_VAR6,
		DISKID_XFORM2_COEF0,			// actual coefficients (6 per DISKID_XFORM)
		DISKID_XFORM2_COEF1,
		DISKID_XFORM2_COEF2,
		DISKID_XFORM2_COEF3,
		DISKID_XFORM2_COEF4,
		DISKID_XFORM2_COEF5,
	DISKID_XFORM2_END,					// end of this DISKID_XFORM section...
	DISKID_XFORM3_COMMAND_MENU,
	DISKID_XFORM3_START,				// start of this DISKID_XFORM section...
		DISKID_XFORM3_ENABLED,			// BOOLEAN  master on/off for this DISKID_XFORM
		DISKID_XFORM3_DENSITY,
		DISKID_XFORM3_COLOR,
		DISKID_XFORM3_VAR0,			// percentage of each variation to use (7 per DISKID_XFORM)
		DISKID_XFORM3_VAR1,
		DISKID_XFORM3_VAR2,
		DISKID_XFORM3_VAR3,
		DISKID_XFORM3_VAR4,
		DISKID_XFORM3_VAR5,
		DISKID_XFORM3_VAR6,
		DISKID_XFORM3_COEF0,			// actual coefficients (6 per DISKID_XFORM)
		DISKID_XFORM3_COEF1,
		DISKID_XFORM3_COEF2,
		DISKID_XFORM3_COEF3,
		DISKID_XFORM3_COEF4,
		DISKID_XFORM3_COEF5,
	DISKID_XFORM3_END,					// end of this DISKID_XFORM section...
	DISKID_XFORM4_COMMAND_MENU,
	DISKID_XFORM4_START,				// start of this DISKID_XFORM section...
		DISKID_XFORM4_ENABLED,			// BOOLEAN  master on/off for this DISKID_XFORM
		DISKID_XFORM4_DENSITY,
		DISKID_XFORM4_COLOR,
		DISKID_XFORM4_VAR0,			// percentage of each variation to use (7 per DISKID_XFORM)
		DISKID_XFORM4_VAR1,
		DISKID_XFORM4_VAR2,
		DISKID_XFORM4_VAR3,
		DISKID_XFORM4_VAR4,
		DISKID_XFORM4_VAR5,
		DISKID_XFORM4_VAR6,
		DISKID_XFORM4_COEF0,			// actual coefficients (6 per DISKID_XFORM)
		DISKID_XFORM4_COEF1,
		DISKID_XFORM4_COEF2,
		DISKID_XFORM4_COEF3,
		DISKID_XFORM4_COEF4,
		DISKID_XFORM4_COEF5,
	DISKID_XFORM4_END,					// end of this DISKID_XFORM section...
	DISKID_XFORM5_COMMAND_MENU,
	DISKID_XFORM5_START,				// start of this DISKID_XFORM section...
		DISKID_XFORM5_ENABLED,			// BOOLEAN  master on/off for this DISKID_XFORM
		DISKID_XFORM5_DENSITY,
		DISKID_XFORM5_COLOR,
		DISKID_XFORM5_VAR0,			// percentage of each variation to use (7 per DISKID_XFORM)
		DISKID_XFORM5_VAR1,
		DISKID_XFORM5_VAR2,
		DISKID_XFORM5_VAR3,
		DISKID_XFORM5_VAR4,
		DISKID_XFORM5_VAR5,
		DISKID_XFORM5_VAR6,
		DISKID_XFORM5_COEF0,			// actual coefficients (6 per DISKID_XFORM)
		DISKID_XFORM5_COEF1,
		DISKID_XFORM5_COEF2,
		DISKID_XFORM5_COEF3,
		DISKID_XFORM5_COEF4,
		DISKID_XFORM5_COEF5,
	DISKID_XFORM5_END,					// end of this DISKID_XFORM section...
	DISKID_HUE_ROTATION,
	DISKID_RANDOM_PERIOCITY,
	DISKID_ALPHA_BLEND,
	DISKID_SHOW_ALPHA_ONLY
	// ADD NEW ITEMS HERE  (*not* in the middle)
	// ADD NEW ITEMS HERE  (*not* in the middle)
	// ADD NEW ITEMS HERE  (*not* in the middle)
	// ADD NEW ITEMS HERE  (*not* in the middle)
	// ADD NEW ITEMS HERE  (*not* in the middle)
};
// ***IMPORTANT*** NEVER EVER DELETE ITEMS FROM THE ABOVE LIST!!!
// ***IMPORTANT*** NEVER EVER ADD ITEMS EXCEPT TO THE *END* OF THE ABOVE LIST!!!


#endif // __AEFLAME_H__



