/* Copyright (c) 1992 Scott Draves <spot@cs.cmu.edu>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */


#include <math.h>
#include <gl/gl.h>
#include "libifs.h"

#define CMAP_SIZE 256


void
add_delta(delta, cp, mult)
   real delta[NXFORMS][3][2];
   control_point *cp;
   real mult;
{
   int i, j, k;
   for (i = 0; i < NXFORMS; i++)
      for (j = 0; j < 3; j++)
	 for (k = 0; k < 2; k++)
	    cp->xform[i].c[j][k] += mult * delta[i][j][k];
}

void
random_delta(delta)
   real delta[NXFORMS][3][2];
{
   int i, j, k;
   for (i = 0; i < NXFORMS; i++)
      for (j = 0; j < 3; j++)
	 for (k = 0; k < 2; k++)
	    delta[i][j][k] = random_uniform11() / 1000.0;
}  


void main(int argc, char **argv) {
   int batch_size = 1000;
   int normal_length =  200000;
   int main_counter;

   control_point cp;
   real delta[NXFORMS][3][2];
   point *points;
   cmap cm;

   

   if (1) {
      real p1[2], p2[2];

      srandom(time(0));

      prefsize(800, 800);
      winopen("flame lines");
      RGBmode();
      gconfig();
      ortho2(-1.0, 1.0, -1.0, 1.0);
      cpack(0x00000000);
      clear();

      main_counter = normal_length;
      random_control_point(&cp, variation_random);
      random_delta(delta);
      get_cmap(cmap_random, cm, CMAP_SIZE);
      points = (point *) malloc(sizeof(point) * batch_size);
   }

   while (1) {
      int i, misses;

      points[0][0] = random_uniform11();
      points[0][1] = random_uniform11();
      points[0][2] = 0.0;
      misses = 0;
      iterate(&cp, batch_size, 10, points);
      color(WHITE);
      bgnclosedline();
      for (i = 0; i < batch_size; i++) {
/* 	  printf("%f %f\n", points[i][0], points[i][1]); */


	 if (points[i][0] > -1.0 &&
	     points[i][1] > -1.0 &&
	     points[i][0] <  1.0 &&
	     points[i][1] <  1.0) {
	    int color_index = (int) (points[i][2] * CMAP_SIZE);
	    if (color_index < 0) color_index = 0;
	    else if (color_index > (CMAP_SIZE-1))
	       color_index = CMAP_SIZE-1;
	    
	    c3f(&cm[color_index][0]);
	    v2f(&points[i][0]);
	 } else
	    if (misses++ > batch_size / 2)
	       main_counter = 1;
      }
      endclosedline();

      if (main_counter > 0) {
	 main_counter -= batch_size;
	 if (main_counter <= 0) {
	    color(BLACK);
	    clear();
	    random_control_point(&cp, variation_random);
	    random_delta(delta);
	    get_cmap(cmap_random, cm, CMAP_SIZE);
	    main_counter = normal_length;
	 }
      }
   }
}

